use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::compare::ElispFact;
use crate::types::GetAffiliatedKeywords;
use crate::types::LatexEnvironment;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmLatexEnvironment {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    pub(crate) value: String,
}

to_wasm!(
    WasmLatexEnvironment,
    LatexEnvironment<'s>,
    original,
    wasm_context,
    { WasmAstNode::LatexEnvironment(original) },
    { "latex-environment".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        Ok((
            Vec::new(),
            WasmLatexEnvironment {
                additional_properties,
                value: original.value.to_owned(),
            },
        ))
    }
);
