use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use crate::compare::ElispFact;
use crate::types::Superscript;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmSuperscript {
    #[serde(rename = "use-brackets-p")]
    pub(crate) use_brackets: bool,
}

to_wasm!(
    WasmSuperscript,
    Superscript<'s>,
    original,
    wasm_context,
    { WasmAstNode::Superscript(original) },
    { "superscript".into() },
    {
        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok((
            children,
            WasmSuperscript {
                use_brackets: original.use_brackets,
            },
        ))
    }
);
